<?php

namespace Drupal\monarch_response_utility\EventSubscriber;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\monarch_response_utility\ResponseException;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\ExceptionEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Event subscriber for assisting responses.
 */
final class EventSubscriber implements EventSubscriberInterface {

  /**
   * Constructs an EventSubscriber object.
   */
  public function __construct(
    private readonly EntityTypeManagerInterface $entityTypeManager,
  ) {}

  /**
   * Kernel response event handler.
   */
  public function onException(ExceptionEvent $event): void {
    $throwable = $event->getThrowable();

    if ($throwable instanceof ResponseException) {
      $event->allowCustomResponseCode();
      $event->setResponse($throwable->toResponse());
      $event->stopPropagation();
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      KernelEvents::EXCEPTION => ['onException'],
    ];
  }

}
